// FarPipeRPCServer.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"
#include <plugin.hpp>
#include <InitGuid.h>

#include <stdio.h>
#include "PipeServer.h"
#include "debug.h"


// {5A7FCDFC-C294-4404-853E-B05EB701F019}
DEFINE_GUID(MainGuid, 0x5a7fcdfc, 0xc294, 0x4404, 0x85, 0x3e, 0xb0, 0x5e, 0xb7, 0x01, 0xf0, 0x19);


// Variables that are initialised when the plugin is loaded
static PluginStartupInfo FarStartupInfo;

// The server that receives and processes commands
static PipeServer* pipeServer;

// Handle of the thread the server runs in
HANDLE pipeServerThreadHandle;


// Data to pass to the pipe server thread
struct ThreadData
{
    PipeServer* pipeServer;
};

// The server thread function
DWORD WINAPI PipeServerThreadFunction(LPVOID lpParam)
{
    ThreadData* threadData = (ThreadData*) lpParam;

    threadData->pipeServer->start(); // Blocks until the exit command is received

    return 0; 
} 


//
// Functions exported by the plugin
//

void WINAPI GetGlobalInfoW(struct GlobalInfo *Info)
{
    Info->StructSize = sizeof(struct GlobalInfo);
    Info->MinFarVersion = MAKEFARVERSION(3, 0, 0, 4000, VS_RELEASE);
    Info->Version = MAKEFARVERSION(1, 0, 0, 0, VS_ALPHA);
    Info->Guid = MainGuid;
    Info->Title = L"FarMgrPipeRPCServer";
    Info->Description = L"Processes requests made via a named pipe";
    Info->Author = L"fml2";
}


void WINAPI SetStartupInfoW(const struct PluginStartupInfo *Info)
{
    FarStartupInfo = *Info;

    DWORD currentPid = GetCurrentProcessId();

    wchar_t pipeName[40];
    swprintf_s(pipeName, sizeof(pipeName)/sizeof(pipeName[0]), L"FarMgrRpcPipe_%d", currentPid);

    logDebug(L"Creating a server for pipe '%s'", pipeName);

    pipeServer = new PipeServer(pipeName, &FarStartupInfo);

    pipeServerThreadHandle = INVALID_HANDLE_VALUE;

    struct ThreadData threadData;
    threadData.pipeServer = pipeServer;

    logDebug(L"Starting the thread for pipe '%s'...", pipeName);

    // Create and start the server thread
    DWORD threadId;
    pipeServerThreadHandle = CreateThread(NULL, 0, PipeServerThreadFunction, &threadData, 0, &threadId);
}

void WINAPI ExitFARW(const struct ExitInfo *Info)
{
    // Set the 'exit requested' flag so that the server leaves its main processing loop
    // thus letting the thread to terminate.
    logDebug(L"Requesting the server stop for the pipe %s", pipeServer->getSimplePipeName().c_str());
    pipeServer->requestExit();

    if (pipeServerThreadHandle != INVALID_HANDLE_VALUE)
    {
        WaitForSingleObject(pipeServerThreadHandle, 5000);
    }

    CloseHandle(pipeServerThreadHandle);
    pipeServerThreadHandle = INVALID_HANDLE_VALUE;
    delete pipeServer;
}
