#include "StdAfx.h"
#include "PipeClient.h"
#include "debug.h"


bool PipeClient::executeCommand(std::wstring &simplePipeName, bool resultIsExpected,
    void *cmdResultBuffer, size_t cmdResultBufferSize, size_t *actualCmdResultSize)
{
    std::wstring pipeName = L"\\\\.\\pipe\\" + simplePipeName;
    const wchar_t *pipeCStr = pipeName.c_str();

    bool pipeConnected = false;
    bool returnValue = false; // Assume the worst

    HANDLE hPipe = CreateFile(
                            pipeCStr,                     // lpFileName
                            GENERIC_READ | GENERIC_WRITE, // dwDesiredAccess
                            0,							  // dwShareMode
                            NULL,						  // lpSecurityAttributes	
                            OPEN_EXISTING,				  // dwCreationDisposition
                            0,							  // dwFlagsAndAttributes
                            NULL);						  // hTemplateFile
    if (hPipe == INVALID_HANDLE_VALUE) {
        logDebug(L"Could not connect to the pipe '%s', error code: %d", pipeCStr, GetLastError());
        goto FunctionEnd;
    }
    pipeConnected = true;
    logDebug(L"Connected to the pipe '%s', handle: %d", pipeCStr, hPipe);

    DWORD writtenBytesCount;

    // Send the command itself
    COMMAND_CODE_SEND_TYPE cmdValue = (COMMAND_CODE_SEND_TYPE) m_command;
    WriteFile(hPipe, &cmdValue, sizeof(cmdValue), &writtenBytesCount, NULL);
    if (writtenBytesCount != sizeof(cmdValue))
    {
        logDebug(L"Could not send the command to the pipe %s: %d instead of %d bytes written, error code: %d",
                pipeCStr, writtenBytesCount, sizeof(cmdValue), GetLastError());
        goto FunctionEnd;
    }
    logDebug(L"Wrote %d bytes of the command code to the pipe", writtenBytesCount);

    // Send the additional data (if present)
    if (m_additionalData != NULL && m_additionalDataSize > 0)
    {
        WriteFile(hPipe, &m_additionalData, m_additionalDataSize, &writtenBytesCount, NULL);
        if (writtenBytesCount != m_additionalDataSize)
        {
            logDebug(L"Could not send the additional data to the pipe %s: %d instead of %d bytes written, error code: %d",
                    pipeCStr, writtenBytesCount, m_additionalDataSize, GetLastError);
            goto FunctionEnd;
        }
        logDebug(L"Wrote %d bytes of additional data to the pipe", writtenBytesCount);
    }

    if (resultIsExpected)
    {
        logDebug(L"Reading results from the pipe %s...", pipeCStr);
        DWORD readBytesCount;
        BOOL succesfullyRead = ReadFile(hPipe, cmdResultBuffer, cmdResultBufferSize, &readBytesCount, NULL);
        if (!succesfullyRead) {
            logDebug(L"Could not read data, error code: %d", GetLastError());
            goto FunctionEnd;
        }
        logDebug(L"Read %d bytes from the pipe: '%s'", readBytesCount, pipeCStr);
    }

    returnValue = true; // If we got here, everything went well

FunctionEnd:
    if (pipeConnected)
    {
        CloseHandle(hPipe);
        pipeConnected = false;
    }

    return returnValue;

}
