#pragma once

#include <string>
#include "Command.h"

// Can send a command with some additional data (parameters) to a named pipe
// (to a server) and read the results from the pipe
class PipeClient
{
public:
	PipeClient(Command command, void *additionalData = NULL, size_t additionalDataSize = 0) :
	    m_command(command), m_additionalData(additionalData), m_additionalDataSize(additionalDataSize)
	{
		// Intentionally left blank
	}

	~PipeClient(void)
	{
		// Intentionally left blank
	}

	// Sends the specified command to the server and reads the results
	bool sendCommandAndGetResult(std::wstring &simplePipeName, void *cmdResultBuffer, size_t cmdResultBufferSize, size_t *actualCmdResultSize)
	{
		return executeCommand(simplePipeName, true, cmdResultBuffer, cmdResultBufferSize, actualCmdResultSize);
	}

	// Sends the specified command which is not expected to produce any results
	bool sendCommandWithoutResult(std::wstring &simplePipeName)
	{
		return executeCommand(simplePipeName, false, NULL, 0, NULL);
	}

private:

	// The command to send
	Command m_command;

	// Data (parameters) to send along the command
	void *m_additionalData;
	size_t m_additionalDataSize;

	// Sends the command to the specified pipe and reads the results, if necessary
	bool executeCommand(std::wstring &simplePipeName, bool resultIsExpected,
		void *cmdResultBuffer, size_t cmdResultBufferSize, size_t *actualCmdResultSize);
};
