#include "stdafx.h"
#include "PipeServer.h"
#include "debug.h"
#include <stdio.h>
#include <string>
#include "Command.h"
#include "PipeClient.h"


PipeServer::PipeServer(wchar_t* pipeSimpleName, PluginStartupInfo* farStartupInfo)
{
    m_simplePipeName = std::wstring(pipeSimpleName);
    m_farStartupInfo = farStartupInfo;
    m_pipeHandle = INVALID_HANDLE_VALUE;
}


PipeServer::~PipeServer(void)
{
}

const std::wstring& PipeServer::getSimplePipeName()
{
    return m_simplePipeName;
}


// Size of the buffer for pipe communication, in bytes
#define PIPE_BUFFER_SIZE 4096


bool PipeServer::start()
{
    std::wstring pipeName = L"\\\\.\\pipe\\" + m_simplePipeName;

    logDebug(L"Creating the pipe '%s'...", pipeName.c_str());

    m_pipeHandle = CreateNamedPipe(pipeName.c_str(), PIPE_ACCESS_DUPLEX, PIPE_TYPE_BYTE,
        1, PIPE_BUFFER_SIZE, PIPE_BUFFER_SIZE, 5000, NULL);

    if (m_pipeHandle == INVALID_HANDLE_VALUE) {
        logDebug(L"Could not create the pipe");
        return false;
    }

    logDebug(L"Created the pipe '%s', handle: %d", pipeName.c_str(), m_pipeHandle);

    bool pipeIsConnected = false;

    // Enter the main loop: receive and execute a command
    m_exitRequested = false;

    while (!m_exitRequested)
    {
        if (pipeIsConnected)
        {
            // Ensure that the client has read all the data
            BOOL sucessfullyFlushed = FlushFileBuffers(m_pipeHandle); // A blocking call!
            if (!sucessfullyFlushed) {
                logDebug(L"Could not flush data to the pipe, error code: %x", GetLastError());
                continue;
            }

            BOOL succesfullyDisconnected = DisconnectNamedPipe(m_pipeHandle);

            if (!succesfullyDisconnected) {
                logDebug(L"Could not disconnect, error code: %x", GetLastError());
                continue;
            }
            pipeIsConnected = false;
        }

        logDebug(L"Waiting for a command...");
        BOOL successfullyConnected = ConnectNamedPipe(m_pipeHandle, NULL);

        if (!successfullyConnected) {
            logDebug(L"Could not connect to the pipe, error code: %x", GetLastError());
            continue;
        }

        pipeIsConnected = true;

        logDebug(L"Connected to the pipe '%s', reading data...", pipeName.c_str());

        char receivedData[PIPE_BUFFER_SIZE];
        DWORD readBytesCount;
        BOOL succesfullyRead = ReadFile(m_pipeHandle, receivedData, PIPE_BUFFER_SIZE, &readBytesCount, NULL);
        
        if (!succesfullyRead) {
            logDebug(L"Could not read data, error code: %x", GetLastError());
            continue;
        }

        logDebug(L"Read %d bytes of command data", readBytesCount);

        COMMAND_CODE_SEND_TYPE commandCode = *((COMMAND_CODE_SEND_TYPE*) receivedData);

        logDebug(L"Command: %d", commandCode);

        if (commandCode == CMD_EXIT)
        {
            m_exitRequested = true;
            logDebug(L"Received the EXIT command");
            continue;
        }

        //
        // Write the response
        //

        DWORD writtenBytesCount;
        BOOL succesfullyWritten = WriteFile(m_pipeHandle, receivedData, readBytesCount, &writtenBytesCount, NULL);

        if (!succesfullyWritten) {
            logDebug(L"Could not write data, error code: %x", GetLastError());
        }

        logDebug(L"Wrote %d bytes to the pipe", writtenBytesCount);
    } // main while loop

    CloseHandle(m_pipeHandle);

    logDebug(L"Closed the pipe hande");

    return true;

}

void PipeServer::requestExit()
{
    // Send an EXIT command to ourselves so that the main loop is left
    PipeClient client = PipeClient(CMD_EXIT);
    client.sendCommandWithoutResult(m_simplePipeName);
}
