#pragma once

#include "stdafx.h"
#include <plugin.hpp>
#include <string>


// Sets up a pipe and then listens to it. If a command is received, it gets executed.
// This repeats until an exit command is received.
class PipeServer
{
private:
	std::wstring m_simplePipeName;

	// Handle to the pipe through which the server receives commands
	HANDLE m_pipeHandle;

	// Pointer to the Far API
	PluginStartupInfo* m_farStartupInfo;

	// Whether an exit from the main loop has been requested (only important
	// after start() has been called)
	bool m_exitRequested;

public:
	// Creates a server that will receive commands throu the pipe with the given simple name,
	// i.e. without the 'technical' name prefix (\\.\pipe\)
	PipeServer(wchar_t* pipeSimpleName, PluginStartupInfo* farStartupInfo);

	~PipeServer();

	const std::wstring& getSimplePipeName();

	// Starts the server, i.e. sets up the pipe, receives and executes commands through it
	// Returns TRUE if averything went OK.
	// The server receives and executes commands on the same thread it was started from.
	// The function exits only after the server has received the EXIT command.
	bool start();

	// Requests stopping of command processing. The command currently being processed
	// gets processed, but then no further command will be accepted
	void requestExit();

};
